package pl.edu.icm.synat.process.common.enrich.impl;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import com.google.common.base.CharMatcher;

import pl.edu.icm.model.bwmeta.y.YAffiliation;
import pl.edu.icm.model.bwmeta.y.YAttribute;
import pl.edu.icm.model.bwmeta.y.YContributor;
import pl.edu.icm.model.bwmeta.y.YElement;
import pl.edu.icm.model.bwmeta.y.YName;
import pl.edu.icm.model.bwmeta.y.constants.attributes.CommonAttributeTypes;
import pl.edu.icm.synat.process.common.enrich.MetadataTransformer;

public class AffiliationDataEnricher implements MetadataTransformer {

    private static final List<String> PATTERNS_TO_REMOVE = Arrays.asList("E[-]?mail:[a-zA-Z0-9._%+-@.]+( |\n)","[\n[0-9]\\., ]+$",
            "( |\n)*[_A-Za-z0-9-\\+]+(\\.[_A-Za-z0-9-]+)*@[A-Za-z0-9-]+(\\.[A-Za-z0-9]+)*(\\.[A-Za-z]{2,})+( |\n|$)");

    @Override
    public boolean modify(YElement element) {
        boolean modify = false;
        Iterator<YAffiliation> affItr = element.getAffiliations().iterator();
        Set<String> removedAffiliations = new HashSet<>();
        while (affItr.hasNext()) {
            YAffiliation aff = affItr.next();
            if (StringUtils.isBlank(aff.getSimpleText()) || CharMatcher.JAVA_LETTER.matchesNoneOf(aff.getSimpleText())) {
                removedAffiliations.add(aff.getId());
                affItr.remove();
                modify = true;
            }
        }
        for (YContributor contributor : element.getContributors()) {
            if (CollectionUtils.isNotEmpty(removedAffiliations) && CollectionUtils.isNotEmpty(contributor.getAffiliationRefs())) {
                contributor.getAffiliationRefs().removeAll(removedAffiliations);
            }
            for (YName name : contributor.getNames()) {
                String cleared = name.getText();
                for (String pattern : PATTERNS_TO_REMOVE) {
                    cleared = StringUtils.removePattern(cleared, pattern);
                }
                if (!StringUtils.equals(cleared, name.getText())) {
                    name.setText(cleared.trim());
                    modify = true;
                }
            }
            final String email = contributor.getOneAttributeSimpleValue(CommonAttributeTypes.AT_CONTACT_EMAIL);
            if (StringUtils.isNotBlank(email)) {
                final String newEmail = StringUtils.removePattern(email, "E[-]?mail:");
                if (!StringUtils.equals(email, newEmail)) {
                    Iterator<YAttribute> itr = contributor.getAttributes().iterator();
                    while (itr.hasNext()) {
                        if (CommonAttributeTypes.AT_CONTACT_EMAIL.equals(itr.next().getKey())) {
                            itr.remove();
                        }
                    }
                    contributor.addAttribute(CommonAttributeTypes.AT_CONTACT_EMAIL, newEmail);
                    modify = true;
                }
            }
        }

        return modify;
    }

}
