package pl.edu.icm.synat.process.common.enrich.impl;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Required;

import pl.edu.icm.model.bwmeta.y.YAncestor;
import pl.edu.icm.model.bwmeta.y.YElement;
import pl.edu.icm.model.bwmeta.y.YStructure;
import pl.edu.icm.model.bwmeta.y.constants.HierarchyWithLevelIds;
import pl.edu.icm.synat.api.services.index.fulltext.query.criteria.impl.FieldCriterion;
import pl.edu.icm.synat.logic.index.publication.PublicationIndexFieldConstants;
import pl.edu.icm.synat.logic.utils.IndexSearchUtil;
import pl.edu.icm.synat.process.common.enrich.MetadataTransformer;

public class IndexStructureEnricher implements MetadataTransformer {
    private static final Logger LOGGER = LoggerFactory.getLogger(IndexStructureEnricher.class);

    private IndexSearchUtil indexSearchUtil;

    private static final String FIELD_NAME = PublicationIndexFieldConstants.FIELD_NAME;

    public String resolveJournalId(final String journalName) {
        if (StringUtils.isBlank(journalName)) {
            return null;
        }
        return indexSearchUtil.resolveOneId(new FieldCriterion(FIELD_NAME, journalName));
    }

    @Override
    public boolean modify(YElement element) {
        if (element != null) {
            YStructure structure = element.getStructure(HierarchyWithLevelIds.EXT_HIERARCHY_JOURNAL);
            if (structure != null) {
                YAncestor journal = structure.getAncestor(HierarchyWithLevelIds.EXT_LEVEL_JOURNAL_JOURNAL);
                if (journal != null && StringUtils.isBlank(journal.getIdentity()) && journal.getOneName() != null ) {
                    final String name = journal.getOneName().getText();
                    final String journalId = resolveJournalId(name);
                    if (journalId != null) {
                        journal.setIdentity(journalId);
                        Map<String,String> parameters = buildParameters(structure);
                        Map<String,String> structureIds = indexSearchUtil.resolveStructureIds(journalId, parameters);
                        if (MapUtils.isNotEmpty(structureIds)) {
                            for (Map.Entry<String, String> entry : structureIds.entrySet()) {
                                structure.getAncestor(entry.getKey()).setIdentity(entry.getValue());
                            }
                        }
                        return true;
                    }
                }
            }
        }
        return false;
    }

    private Map<String, String> buildParameters(YStructure structure) {
        Map<String,String> output = new HashMap<String, String>();
        appendAncestorName(structure, output,HierarchyWithLevelIds.EXT_LEVEL_JOURNAL_YEAR);
        appendAncestorName(structure, output,HierarchyWithLevelIds.EXT_LEVEL_JOURNAL_VOLUME);
        appendAncestorName(structure, output,HierarchyWithLevelIds.EXT_LEVEL_JOURNAL_ISSUE);
        return output;
    }

    private void appendAncestorName(YStructure structure, Map<String, String> output, String level) {
        YAncestor ancestor = structure.getAncestor(level);
        if (ancestor != null && ancestor.getOneName()!=null ) {
            final String name = ancestor.getOneName().getText();
            if (StringUtils.isNotBlank(name)) {
                output.put(level, name);
            }
        }
    }

    @Required
    public void setIndexSearchUtil(IndexSearchUtil indexSearchUtil) {
        this.indexSearchUtil = indexSearchUtil;
    }

}
