package pl.edu.icm.synat.process.common.enrich.impl.sonca;

/*
 * #%L
 * Synat: 
 * %%
 * Copyright (C) 2010 - 2013 ICM, Warsaw University
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;

import pl.edu.icm.model.bwmeta.y.YElement;
import pl.edu.icm.model.bwmeta.y.YExportable;
import pl.edu.icm.model.transformers.MetadataWriter;
import pl.edu.icm.model.transformers.bwmeta.y.BwmetaTransformerConstants;
import pl.edu.icm.synat.application.model.bwmeta.transformers.SynatMetadataTransformers;
import pl.edu.icm.synat.application.model.bwmeta.utils.BwmetaConverterUtils;
import pl.edu.icm.synat.process.common.enrich.MetadataEnricher;
import pl.edu.icm.synat.process.common.enrich.type.SoncaEnrichType;

/**
 * {@code Sonca/OECD classification} based meta data enricher.
 * Implementation of {@code MetadataEnricher}.
*
* @author Gra <Gołębiewski Radosław A.> google.com/+RadoslawGolebiewski
*
*/
public class SoncaMetadataEnricher implements MetadataEnricher {
	/**
	 * Default OECD_ONLY enriching is used.
	 */
    private SoncaEnrichType enrichingtype = SoncaEnrichType.OECD_ONLY;
    private String url;
    private RestTemplate restTemplate;

    public SoncaMetadataEnricher(RestTemplate restTemplate, String url, SoncaEnrichType enrichingtype) {
        this.url = url;
        this.restTemplate = restTemplate;
        this.enrichingtype = enrichingtype;
    }

    public SoncaMetadataEnricher(RestTemplate restTemplate, String url) {
        this.url = url;
        this.restTemplate = restTemplate;
    }

    public SoncaMetadataEnricher() {
    }

    @Override
    public String enrich(String bwmeta) {
        final ResponseEntity<byte[]> response = restTemplate.exchange(
                new StringBuilder(url).append(enrichingtype.getUrlSufix()).toString()
                   , HttpMethod.PUT, prepareRequest(bwmeta), byte[].class);
        byte[] bytes = response.getBody();
        try {
			return new String(bytes, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			return null;
		}
    }

    @Override
    public YElement enrich(final List<YElement> elements) {
        String enrichedBwmeta = enrich(prepareBwmeta(elements));
        return BwmetaConverterUtils.bwmetaToYElement(enrichedBwmeta);
    }

    private String prepareBwmeta(final List<YElement> elements) {
        MetadataWriter<YExportable> writer = SynatMetadataTransformers.BTF.getWriter(BwmetaTransformerConstants.Y
        		, BwmetaTransformerConstants.BWMETA_2_1);
        return writer.write(prepareYExportableList(elements));
    }

    private List<YExportable> prepareYExportableList(
            final List<YElement> elements) {
        List<YExportable> list = new ArrayList<YExportable>();
        list.addAll(elements);
        return list;
    }

    private HttpEntity<String> prepareRequest(String bwmeta) {
        final HttpHeaders requestHeaders = new HttpHeaders();
        requestHeaders.set("Content-type", MediaType.APPLICATION_XML_VALUE);
        final HttpEntity<String> requestEntity = new HttpEntity<>(bwmeta, requestHeaders);
        return requestEntity;
    }

    public void setEnrichingtype(SoncaEnrichType enrichingtype) {
        this.enrichingtype = enrichingtype;
    }

    public void setUrl(String url) {
        this.url = url;
    }

    public void setRestTemplate(RestTemplate restTemplate) {
        this.restTemplate = restTemplate;
    }

    public SoncaEnrichType getEnrichingtype() {
        return enrichingtype;
    }
}
