package pl.edu.icm.synat.process.common.enrich.node;

/*
 * #%L
 * Synat: License data importer.
 * %%
 * Copyright (C) 2010 - 2013 ICM, Warsaw University
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import java.util.List;

import pl.edu.icm.model.bwmeta.y.YElement;
import pl.edu.icm.model.bwmeta.y.YExportable;
import pl.edu.icm.synat.api.services.process.problem.ProblemHandler;
import pl.edu.icm.synat.api.services.process.stats.LogSeverity;
import pl.edu.icm.synat.logic.document.model.api.Document;
import pl.edu.icm.synat.process.common.enrich.MetadataTransformer;
import pl.edu.icm.synat.process.common.node.SkippableNode;

/**
 * Enrich metadata by oecd discipline. Process if element exists and if
 * {@code MetadataTransformer} modify return true. Then sets metadata changed.
 * Implements {@code ItemProcessor}.
 *
 * @author Gra <Gołębiewski Radosław A.> google.com/+RadoslawGolebiewski
 *
 */
public class MetadataEnricherNode extends SkippableNode<List<Document>> {

    private MetadataTransformer transformer;
    private ProblemHandler problemHandler;

    /**
     * @param processingEnabled, enable or disable processing.
     */
    public MetadataEnricherNode(Boolean processingEnabled) {
        this.processingEnabled = processingEnabled != null && processingEnabled;
    }

    @Override
    public List<Document> innerProcess(final List<Document> input) {
        for (Document document : input) {
            processDocument(document);
        }
        return input;
    }

    private void processDocument(Document document) {
        YExportable exportable = document.getMetadata();
        if (!(exportable instanceof YElement)) {
            return;
        }
        YElement element = (YElement) exportable;

        try {
            if (transformer.modify(element)) {
                document.setMetadata(exportable);
            }
        } catch (Exception e) {
            problemHandler.handleProblem(LogSeverity.WARN, document.getId(), "enriching problem", e);
        }
    }

    public void setTransformer(MetadataTransformer transformer) {
        this.transformer = transformer;
    }

    public void setProblemHandler(ProblemHandler problemHandler) {
        this.problemHandler = problemHandler;
    }
}
