package pl.edu.icm.synat.process.common.harvesting.metadata.utils.preapre.content;

import java.io.IOException;

import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.Cacheable;

public class PrepareContentUtilsImpl implements PrepareContentUtils {
	
    private static final Logger LOGGER = LoggerFactory.getLogger(PrepareContentUtils.class);

    @Override
    @Cacheable("remoteContent")
    public String fetchRemoteContent(String path) {
        if (null == path) {
            return null;
        }
        GetMethod method = null;
        try {
            HttpClient httpClient = new HttpClient();
            method = new GetMethod(path);
            method.getParams().setParameter(HttpMethodParams.RETRY_HANDLER
                    , new DefaultHttpMethodRetryHandler(3, false));
            // Execute the method.
            int statusCode = httpClient.executeMethod(method);

            if (statusCode != HttpStatus.SC_OK) {
                LOGGER.info("Method failed: {}, {}.", path, method.getStatusLine());
            } else {
                return IOUtils.toString(method.getResponseBodyAsStream());
            }
        } catch (HttpException e) {
            LOGGER.info("http error on {}.", path, e);
        } catch (IOException e) {
            LOGGER.info("io error on {}.", path, e);
        } finally {
            if (method != null) {
                method.releaseConnection();
            }
        }
        return null;
    }

}
