package pl.edu.icm.synat.process.common.harvesting.metadata.utils.preapre.yelement;

/*
 * #%L
 * Synat: .
 * %%
 * Copyright (C) 2010 - 2013 ICM, Warsaw University
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import pl.edu.icm.model.bwmeta.y.YContentEntry;
import pl.edu.icm.model.bwmeta.y.YContentFile;
import pl.edu.icm.model.bwmeta.y.YElement;
import pl.edu.icm.model.bwmeta.y.YExportable;
import pl.edu.icm.model.bwmeta.y.constants.FileTypes;
import pl.edu.icm.model.bwmeta.y.constants.HierarchyWithLevelIds;
import pl.edu.icm.synat.hierarchy.utils.HierarchyUtils;
import pl.edu.icm.synat.logic.document.model.api.Document;

/**
 * YElement preparing Utils for Metadata harvesting.
 * Process {@code YElement} and {@code Document} for cover and {@code YElement}.
*
* @author Gra <Gołębiewski Radosław A.> google.com/+RadoslawGolebiewski
*
*/
public class PrepareYElementUtils {
    /**
     * Checks if the {@code YElement} has {@code FT_COVER} of {@code FileTypes}.
     * @param element, {@code YElement} used to validation.
     * @return, true if {@code YElement} has correct structure and cover is present,
     *  or false otherwise.
     */
    @SuppressWarnings("rawtypes")
    public boolean hasCover(YElement element) {
        if (null == element) {
            return false;
        }
        for (YContentEntry entry : element.getContents()) {
            if ( ! entry.isFile()) {
                continue;
            }
            YContentFile contentFile = (YContentFile)entry;
            if (FileTypes.FT_COVER.equals(contentFile.getType())) {
                return true;
            }
        }
        return false;
    }

    /**
     * Extracts {@code YElement} from {@code Document}.
     * @param document, {@code Document} used to {@code YElement} extraction.
     * @return, null if the {@code YElement} could not be extracted from given {@code Document}.
     *  Or extracted {@code YElement}.
     */
    public YElement prepareYElement(Document document) {
        YExportable exportable = document.getMetadata();
        if ( ! (exportable instanceof YElement)) {
            return null;
        }
        return (YElement) exportable;
    }

    /**
     * For given {@code Document} and provider check is {@code Document} applicable for adding
     * each metadata. Considered are: {@code Document} and provider compatibility,
     *  lack of description ie. abstract, lack of cover and is {@code Document} considered as
     *  journal or book.
     * @param document, document witch be checked.
     * @param provider, {@code String} representation of the provider.
     * @param withoutBooks, if true then only journal case is checked, if false then books and journals are checked.
     * @return false if document no need to be updated or true if updating the document is needed. 
     */
    public boolean isApplicableFor(Document document, String provider, boolean withoutBooks) {
        String dataset = document.getTagValue("dataset");
        if ( dataset == null || !dataset.equalsIgnoreCase(provider)) {
            return false;
        }
        YElement element = prepareYElement(document);
        if (null == element) {
            return false;
        }
        if ( ! element.getDescriptions().isEmpty() && hasCover(element)) {
            return false;
        }

        if (withoutBooks) {
            return HierarchyUtils.isOnElementLevel(element, HierarchyWithLevelIds.EXT_LEVEL_JOURNAL_JOURNAL);
        }

        return HierarchyUtils.isOnElementLevel(element, HierarchyWithLevelIds.EXT_LEVEL_JOURNAL_JOURNAL)
               || HierarchyUtils.isOnElementLevel(element, HierarchyWithLevelIds.EXT_LEVEL_BOOK_BOOK);
    }
}
