package pl.edu.icm.synat.process.common.node;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.batch.core.JobExecution;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.job.flow.FlowExecutionStatus;
import org.springframework.batch.core.job.flow.JobExecutionDecider;

import pl.edu.icm.synat.application.model.bwmeta.utils.StructureDataUtil;
import pl.edu.icm.synat.logic.services.repository.constants.RepositoryStoreConstants;

public class BuildStructureShouldEndDecider implements JobExecutionDecider {
    protected Logger logger = LoggerFactory.getLogger(BuildStructureShouldEndDecider.class);

    private final static String CONTINUE = "CONTINUE";
    private final StructureDataUtil structureDataUtil;

    public BuildStructureShouldEndDecider(StructureDataUtil structureDataUtil) {
        super();
        this.structureDataUtil = structureDataUtil;
    }

    @Override
    public FlowExecutionStatus decide(JobExecution jobExecution, StepExecution stepExecution) {
        final String currentLevel = jobExecution.getExecutionContext().getString(RepositoryStoreConstants.BUILD_STRUCTURE_CURRENT_LEVEL, null);
        final String nextLevel = structureDataUtil.fetchNext(currentLevel);
        if (nextLevel == null) {
            return FlowExecutionStatus.COMPLETED;
        }

        logger.debug("Decider next processing of {}", nextLevel);
        jobExecution.getExecutionContext().putString(RepositoryStoreConstants.BUILD_STRUCTURE_CURRENT_LEVEL, nextLevel);
        return new FlowExecutionStatus(CONTINUE);
    }

}
