package pl.edu.icm.synat.process.common.node;

/*
 * #%L
 * Synat: process common
 * %%
 * Copyright (C) 2010 - 2013 ICM, Warsaw University
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import java.util.List;

import org.springframework.batch.item.ItemProcessor;
import org.springframework.batch.item.ItemWriter;

import pl.edu.icm.synat.api.services.process.problem.ProblemHandler;
import pl.edu.icm.synat.api.services.process.stats.LogSeverity;
import pl.edu.icm.synat.application.exception.BwmetaValidationException;
import pl.edu.icm.synat.application.model.bwmeta.validators.BwmetaValidator;
import pl.edu.icm.synat.process.common.model.ElementRecord;

/**
 * Node validates YElement
 * 
 * @author lukwas
 */
public class BwmetaValidationNode implements ItemProcessor<ElementRecord, ElementRecord>, ItemWriter<ElementRecord> {
    private final List<BwmetaValidator> validators;
    private final ProblemHandler problemHandler;

    public BwmetaValidationNode(final List<BwmetaValidator> validators, ProblemHandler problemHandler) {
        super();
        this.validators = validators;
        this.problemHandler = problemHandler;
    }

    @Override
    public ElementRecord process(final ElementRecord element) {
        for (BwmetaValidator validator : validators) {
            try {
                validator.validate(element.getyElement());
            } catch (BwmetaValidationException e) {
                problemHandler.handleProblem(LogSeverity.WARN, element.getSourceRecord().getIdentifier().getUid(), e.getValidationType(), e);
            }
        }
        return element;
    }

    @Override
    public void write(List<? extends ElementRecord> items) {
        for (ElementRecord elementRecord : items) {
            process(elementRecord);
        }
    }
}
