package pl.edu.icm.synat.process.common.node;

/*
 * #%L
 * Synat: process common
 * %%
 * Copyright (C) 2010 - 2013 ICM, Warsaw University
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import org.springframework.batch.item.ItemProcessor;

import pl.edu.icm.model.bwmeta.y.YElement;
import pl.edu.icm.model.bwmeta.y.YExportable;
import pl.edu.icm.synat.api.services.process.problem.ProblemHandler;
import pl.edu.icm.synat.api.services.process.stats.LogSeverity;
import pl.edu.icm.synat.application.exception.BwmetaValidationException;
import pl.edu.icm.synat.logic.document.model.api.Document;
import pl.edu.icm.synat.logic.repository.YElementEnricher;

/**
 * Node gets additional information about YElement's parents
 * 
 * @author lukwas
 */
public class DocumentEnricherNode implements ItemProcessor<Document, Document> {
    private final YElementEnricher structureBuilder;
    private final ProblemHandler problemHandler;

    public DocumentEnricherNode(final YElementEnricher structureBuilder, ProblemHandler problemHandler) {
        super();
        this.structureBuilder = structureBuilder;
        this.problemHandler = problemHandler;
    }

    @Override
    public Document process(final Document document) {
        try {
            YExportable metadata = document.getMetadata();
            if (metadata instanceof YElement) {
                YElement element = (YElement) metadata;
                if (structureBuilder.enrich(element)) {
                    document.setMetadata(element);
                }
            }else{
                problemHandler.handleProblem(LogSeverity.WARN, document.getId(), "YExportable", "metadata not YElement");
            }
        } catch (BwmetaValidationException e) {
            problemHandler.handleProblem(LogSeverity.WARN, document.getId(), e.getValidationType(), e);
        }
        return document;
    }
}
