package pl.edu.icm.synat.process.common.node.processor;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.batch.item.ItemProcessor;

import pl.edu.icm.model.bwmeta.y.YContributor;
import pl.edu.icm.model.bwmeta.y.YElement;
import pl.edu.icm.model.bwmeta.y.YExportable;
import pl.edu.icm.synat.api.neo4j.people.model.ContentIndexDocumentWithIdentities;
import pl.edu.icm.synat.api.neo4j.people.model.ContentType;
import pl.edu.icm.synat.api.neo4j.people.model.IdentityIndexDocument;
import pl.edu.icm.synat.api.neo4j.people.model.PeopleIndexDocument;
import pl.edu.icm.synat.application.model.bwmeta.utils.BwmetaContributorUtils;
import pl.edu.icm.synat.logic.document.model.api.Document;
import pl.edu.icm.synat.logic.document.model.api.NativeDocument;
import pl.edu.icm.synat.logic.model.utils.YModelUtils;

public class DocumentToPeopleProcessorNode implements ItemProcessor<NativeDocument, List<PeopleIndexDocument>> {

	protected Logger logger = LoggerFactory.getLogger(DocumentToPeopleProcessorNode.class);

	@Override
	public List<PeopleIndexDocument> process(NativeDocument item) throws Exception {
		List<PeopleIndexDocument> result = new ArrayList<>();
		if (!(item instanceof Document)) {
			return result;
		}
		Document document = (Document) item;
		YExportable exportable = document.getMetadata();
		if (!(exportable instanceof YElement)) {
			return result;
		}
		YElement element = (YElement) exportable;
		ContentIndexDocumentWithIdentities contentDocument = new ContentIndexDocumentWithIdentities(item.getId(), ContentType.PUBLICATION);
		for (YContributor contributor : element.getContributors()) {
			String contributorId = BwmetaContributorUtils.buildContributorId(contentDocument.getId(), contributor);
			IdentityIndexDocument indexDocument = new IdentityIndexDocument(contributorId);
			indexDocument.setRole(contributor.getRole());
			String surname = YModelUtils.fetchSurname(contributor);
			indexDocument.setSurname(surname!= null?surname.replaceAll("[^\\w ]", ""):surname);
			String name = YModelUtils.fetchNames(contributor);
			indexDocument.setName(name!= null?name.replaceAll("[^\\w ]", ""):name);
			String identity = BwmetaContributorUtils.getContributorIdentity(contributor);
	        if(StringUtils.isNotBlank(identity)){
				indexDocument.setPersonId(identity);
			}
			contentDocument.getContributors().add(indexDocument);
		}
		result.add(contentDocument);
		return result;
	}

}
