package pl.edu.icm.synat.process.common.node.reader;

/*
 * #%L
 * Synat: BWMETA indexing process definition and implementation
 * %%
 * Copyright (C) 2010 - 2013 ICM, Warsaw University
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.batch.item.ExecutionContext;
import org.springframework.batch.item.ItemStreamException;
import org.springframework.batch.item.ItemStreamReader;
import org.springframework.util.Assert;

import pl.edu.icm.synat.api.services.store.RecordIterator;
import pl.edu.icm.synat.api.services.store.StatelessStore;
import pl.edu.icm.synat.api.services.store.model.Record;
import pl.edu.icm.synat.api.services.store.model.RecordConditions;
import pl.edu.icm.synat.logic.services.repository.constants.RepositoryStoreConstants;

/**
 * Source node iterating by all elements from store simplified version of
 * SimpleRecordReader maybe should be parent
 * 
 * @author lukwas
 */
public class StructureRecordReader implements ItemStreamReader<Record> {
    protected Logger logger = LoggerFactory.getLogger(StructureRecordReader.class);

    private static final int limit = 300;

    protected final StatelessStore store;
    private final String currentLevel;
    private final String definitionId;
    private final String importId;
    
    private RecordIterator recordIterator;

    private boolean opened = false;

    public StructureRecordReader(StatelessStore store, String currentLevel) {
        this(store, currentLevel, null, null);
    }

    public StructureRecordReader(StatelessStore store, String currentLevel, final String definitionId, final String importId) {
        this.store = store;
        this.currentLevel = currentLevel;
        this.definitionId = definitionId;
        this.importId = importId;
        Assert.notNull(store, "Store is null");
    }

    private RecordConditions prepareCondition(final String currentLevel) {
        return new RecordConditions().withTags(RepositoryStoreConstants.TAG_PREFIX_HIERARHY_LEVEL + currentLevel);
    }

    @Override
    public synchronized void open(ExecutionContext executionContext) throws ItemStreamException {
        if (currentLevel != null && !opened) {
            logger.debug("Read records on level {}", currentLevel);
            RecordConditions condition;
            
            condition = prepareCondition(currentLevel);
            if (StringUtils.isNotBlank(importId)) {
                condition.withTags(RepositoryStoreConstants.TAG_PREFIX_IMPORT + importId);
            } else if (StringUtils.isNotBlank(definitionId)) {
        		condition.withTags(RepositoryStoreConstants.TAG_PREFIX_IMPORT_DATA_SOURCE_DEFINITIONS + definitionId);
        	}
            
            this.recordIterator = new RecordIterator(store, condition, limit);
            opened = true;
        }
    }

    @Override
    public void update(ExecutionContext executionContext) throws ItemStreamException {
    }

    @Override
    public void close() throws ItemStreamException {
        opened = false;
    }

    @Override
    public synchronized Record read() {
        if (!opened) {
            throw new IllegalStateException("Reader not opened yet");
        }
        if (recordIterator.hasNext()) {
            return recordIterator.next();
        } else {
            return null;
        }

    }

}
