package pl.edu.icm.synat.process.common.node.writer;

import java.util.List;

import org.springframework.batch.item.ItemWriter;

import pl.edu.icm.synat.api.services.process.problem.ProblemHandler;
import pl.edu.icm.synat.api.services.process.stats.LogSeverity;
import pl.edu.icm.synat.logic.document.model.api.NativeDocument;
import pl.edu.icm.synat.logic.document.repository.DocumentRepository;

public class NativeDocumentWriter implements ItemWriter<NativeDocument> {

	private static final String MONGO_WRITE = "mongo write";
	private DocumentRepository repository;
	private String eventTag;
	private boolean replace = false;
	private ProblemHandler problemHandler;
	
	public void setRepository(DocumentRepository repository) {
		this.repository = repository;
	}
	
	public void setEventTag(String eventTag) {
		this.eventTag = eventTag;
	}
	
	
	public void setReplace(boolean replace) {
		this.replace = replace;
	}
	
	@Override
	public void write(List<? extends NativeDocument> items) throws Exception {
		for (NativeDocument nativeDocument : items) {
			try {
				if(replace){
					repository.replaceDocument(nativeDocument, eventTag);
				}else {
					repository.storeDocument(nativeDocument, eventTag);
				}
			} catch (Exception e){
				problemHandler.handleProblem(LogSeverity.WARN, nativeDocument.getId(), MONGO_WRITE, e);
				throw e;
			}
		}
	}
	
	public void setProblemHandler(ProblemHandler problemHandler) {
		this.problemHandler = problemHandler;
	}

}
