package pl.edu.icm.synat.process.common.node.writer;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.batch.item.ItemWriter;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.dao.ConcurrencyFailureException;

import pl.edu.icm.synat.api.neo4j.people.PeopleIndexService;
import pl.edu.icm.synat.api.neo4j.people.model.PeopleIndexDocument;
import pl.edu.icm.synat.api.services.process.problem.ProblemHandler;
import pl.edu.icm.synat.api.services.process.stats.LogSeverity;
import pl.edu.icm.synat.logic.document.model.api.Document;
import pl.edu.icm.synat.process.common.node.processor.DocumentToPeopleProcessorNode;

public class PeopleDocumentWriter implements ItemWriter<Document> {

	private DocumentToPeopleProcessorNode peopleNodesTransformer;

	private PeopleIndexService service;

	private ProblemHandler problemHandler;
	
	@Override
	public void write(List<? extends Document> items) throws Exception {
		List<PeopleIndexDocument> documents = new ArrayList<>();
		for (Document document : items) {
			documents.addAll(peopleNodesTransformer.process(document));
		}
		try{
            if (CollectionUtils.isNotEmpty(documents)) {
                service.updateDocuments(documents);
            }
		}catch(ConcurrencyFailureException e){
			problemHandler.handleProblem(LogSeverity.ERROR, "Problem when updating people index", e);
			throw e;
		}
		
	}

	@Required
	public void setPeopleNodesTransformer(DocumentToPeopleProcessorNode peopleNodes) {
		this.peopleNodesTransformer = peopleNodes;
	}

	@Required
	public void setService(PeopleIndexService service) {
		this.service = service;
	}
	
	@Required
	public void setProblemHandler(ProblemHandler problemHandler) {
		this.problemHandler = problemHandler;
	}

}
