package pl.edu.icm.synat.process.common.store.impl;

/*
 * #%L
 * Synat: BWMETA indexing process definition and implementation
 * %%
 * Copyright (C) 2010 - 2013 ICM, Warsaw University
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import java.util.List;

import org.springframework.batch.item.ItemWriter;

import pl.edu.icm.synat.api.services.process.problem.ProblemHandler;
import pl.edu.icm.synat.api.services.process.stats.LogSeverity;
import pl.edu.icm.synat.logic.document.model.api.NativeDocument;
import pl.edu.icm.synat.logic.document.repository.DocumentRepository;
import pl.edu.icm.synat.process.common.store.RepositoryWriterListener;

/**
 * 
 * @author Gra <Gołębiewski Radosław A.> google.com/+RadoslawGolebiewski
 */
public class DirectNativeDocumentBaseWriter implements ItemWriter<NativeDocument> {

    private DocumentRepository repository;
    private String eventTag;
    private ProblemHandler problemHandler;
    private RepositoryWriterListener writerListener;

    public void setRepository(DocumentRepository repository) {
        this.repository = repository;
    }

    public void setEventTag(String eventTag) {
        this.eventTag = eventTag;
    }
    
    public void setProblemHandler(ProblemHandler problemHandler) {
        this.problemHandler = problemHandler;
    }

    public void setWriterListener(RepositoryWriterListener writerListener) {
		this.writerListener = writerListener;
	}
    
    @Override
    public void write(List<? extends NativeDocument> items) throws Exception {
        for (NativeDocument document : items) {
            try {
                repository.storeDocument(document, eventTag);
                if (writerListener != null) {
                	writerListener.successfulSaved(document);
                }
            } catch (Exception e) {
                String id; 
                if (document != null) {
                    id = document.getId();
                } else {
                    id = "Unknown";
                }
                problemHandler.handleProblem(LogSeverity.WARN, id, "Writing documents", e);
                if (writerListener != null) {
                	writerListener.errorOnSave(document, e);
                }

            }
        }
    }
}
