package pl.edu.icm.synat.process.common.store.impl;

import java.util.Iterator;

import org.springframework.batch.item.ExecutionContext;
import org.springframework.batch.item.ItemStreamException;
import org.springframework.batch.item.ItemStreamReader;

import com.google.common.base.Function;

import pl.edu.icm.synat.api.services.store.model.Record;
import pl.edu.icm.synat.process.common.store.RecordIteratorBuilder;
import pl.edu.icm.synat.services.process.ProcessManagerConstants;

public class StoreBasedReader<T> implements ItemStreamReader<T> {

	private static final String RESUMPTION_TOKEN_KEY = "recentResumptionToken";
	private Function<Record, T> transformer;
	private RecordIteratorBuilder iteratorBuilder;
	private Iterator<Record> iterator = null;

	public void setTransformer(Function<Record, T> transformer) {
		this.transformer = transformer;
	}
	
	public void setIteratorBuilder(RecordIteratorBuilder iteratorBuilder) {
		this.iteratorBuilder = iteratorBuilder;
	}

	@Override
	public synchronized void open(ExecutionContext executionContext) {
	
		if (iterator == null) {
			if (executionContext.containsKey(RESUMPTION_TOKEN_KEY)) {
				String resumptionToken = executionContext.getString(RESUMPTION_TOKEN_KEY);
				iteratorBuilder.setResumptionToken(resumptionToken);
			}
	
			long expectedSize = iteratorBuilder.calculateExpectedSize();
			executionContext.put(ProcessManagerConstants.EXECUTION_CONTEXT_KEY_ESTIMATED_TOTAL, expectedSize);
	
			iterator = iteratorBuilder.build();
		}
	}

	@Override
	public synchronized void update(ExecutionContext executionContext)
			throws ItemStreamException {
		String lastToken = iteratorBuilder.getLastToken();
		if (lastToken != null) {
			executionContext.put(RESUMPTION_TOKEN_KEY, lastToken);
		}
	}

	@Override
	public void close() throws ItemStreamException {
	}

	@Override
	public T read() {
		
		T result;
		
		do {
			Record readRecord;
			
			readRecord = readRecord();
			if (readRecord == null) {
				return null;
			}
			
			result = transform(readRecord);
		} while (result == null);
		
		return result;
	}

	@SuppressWarnings("unchecked")
	private T transform(Record readRecord) {
		T result;
		if (transformer != null && readRecord != null) {
			result = transformer.apply(readRecord);
		} else {
			result = (T) readRecord;
		}
		return result;
	}

	private Record readRecord() {
		Record readRecord;
		synchronized (iterator) {
			if (!iterator.hasNext()) {
				readRecord = null;
			} else {
				readRecord = iterator.next();
			}
		}
		return readRecord;
	}

}
