package pl.edu.icm.synat.process.common.store.impl;

import java.util.Iterator;

import org.springframework.beans.factory.annotation.Required;

import pl.edu.icm.synat.api.services.common.Page;
import pl.edu.icm.synat.api.services.common.PageIterable;
import pl.edu.icm.synat.api.services.common.PageIterable.PageCallback;
import pl.edu.icm.synat.api.services.store.StoreQueryService;
import pl.edu.icm.synat.api.services.store.model.Record;
import pl.edu.icm.synat.api.services.store.model.StoreQuery;
import pl.edu.icm.synat.process.common.store.RecordIteratorBuilder;

public class StoreQueryIteratorBuilder implements RecordIteratorBuilder {

	private StoreQueryService storeQueryService;

	private StoreQuery storeQuery;

	private Integer startPage = 0;

	private PageIterable<Record> pageIterable;

	@Required
	public void setStoreQuery(StoreQuery storeQuery) {
		this.storeQuery = storeQuery;
	}

	@Required
	public void setStoreQueryService(StoreQueryService storeQueryService) {
		this.storeQueryService = storeQueryService;
	}

	@Override
	public long calculateExpectedSize() {
		return storeQueryService.countRecords(storeQuery);
	}

	@Override
	public Iterator<Record> build() {
		PageCallback<Record> callback = new PageCallback<Record>() {

			private final StoreQuery query = (StoreQuery) storeQuery.clone();

			@Override
			public Page<Record> getPage(Integer pageNo) {
				query.setPageNo(pageNo);
				return storeQueryService.fetchRecords(query);
			}
		};
		PageIterable<Record> pageIterable = new PageIterable<>(callback, startPage);
		return pageIterable.iterator();
	}

	@Override
	public String getLastToken() {
		return pageIterable != null ? pageIterable.getCurrentPage().toString() : "0";
	}

	@Override
	public void setResumptionToken(String token) {
		this.startPage = Integer.parseInt(token);
	}

}
